@@ -4,7 +4,7 @@ Movie module (imdb package).
 This module provides the Movie class, used to store information about
 a given movie.
 
-Copyright 2004-2007 Davide Alberani <da@erlug.linux.it>
+Copyright 2004-2009 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,10 +21,9 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
-from types import UnicodeType, ListType, TupleType, DictType
 from copy import deepcopy
 
-from imdb.utils import analyze_title, build_title, normalizeTitle, \
+from imdb.utils import analyze_title, build_title, canonicalTitle, \
                         flatten, _Container, cmpMovies
 
 
@@ -43,6 +42,7 @@ class Movie(_Container):
 
     # Aliases for some not-so-intuitive keys.
     keys_alias = {
+                'tv schedule': 'airing',
                 'user rating':  'rating',
                 'plot summary': 'plot',
                 'plot summaries': 'plot',
@@ -50,7 +50,6 @@ class Movie(_Container):
                 'created by': 'creator',
                 'writing credits': 'writer',
                 'produced by':  'producer',
-                #'composer': 'original music',
                 'original music by':    'original music',
                 'non-original music by':    'non-original music',
                 'music':    'original music',
@@ -64,15 +63,14 @@ class Movie(_Container):
                 'casting by':   'casting director',
                 'casting':  'casting director',
                 'art direction by': 'art direction',
-                #'art director': 'art direction',
                 'set decoration by':    'set decoration',
-                #'set decorator': 'set decoration',
                 'costume design by':    'costume designer',
                 'costume design':    'costume designer',
                 'makeup department':    'make up',
                 'makeup':    'make up',
                 'make-up':    'make up',
                 'production management':    'production manager',
+                'production company':    'production companies',
                 'second unit director or assistant director':
                                                 'assistant director',
                 'second unit director':   'assistant director',
@@ -80,6 +78,7 @@ class Movie(_Container):
                 'costume and wardrobe department': 'costume department',
                 'special effects by':   'special effects',
                 'visual effects by':    'visual effects',
+                'special effects company':   'special effects companies',
                 'stunts':   'stunt performer',
                 'other crew':   'miscellaneous crew',
                 'misc crew':   'miscellaneous crew',
@@ -88,6 +87,9 @@ class Movie(_Container):
                 'crew members': 'miscellaneous crew',
                 'other companies': 'miscellaneous companies',
                 'misc companies': 'miscellaneous companies',
+                'miscellaneous company': 'miscellaneous companies',
+                'misc company': 'miscellaneous companies',
+                'other company': 'miscellaneous companies',
                 'aka':  'akas',
                 'also known as':    'akas',
                 'country':  'countries',
@@ -109,6 +111,7 @@ class Movie(_Container):
                 'distributor': 'distributors',
                 'distribution': 'distributors',
                 'distribution companies': 'distributors',
+                'distribution company': 'distributors',
                 'guest': 'guests',
                 'guest appearances': 'guests',
                 'tv guests': 'guests',
@@ -169,7 +172,7 @@ class Movie(_Container):
     def set_title(self, title):
         """Set the title of the movie."""
         # XXX: convert title to unicode, if it's a plain string?
-        d_title = analyze_title(title, canonical=1)
+        d_title = analyze_title(title)
         self.data.update(d_title)
 
     def _additional_keys(self):
@@ -188,26 +191,23 @@ class Movie(_Container):
         """Handle special keys."""
         if self.data.has_key('episode of'):
             if key == 'long imdb episode title':
-                return build_title(self.data, canonical=0)
+                return build_title(self.data)
             elif key == 'series title':
-                ser_title = self.data['episode of'].get('canonical title') or \
-                            self.data['episode of']['title']
-                return normalizeTitle(ser_title)
+                return self.data['episode of']['title']
             elif key == 'canonical series title':
-                ser_title = self.data['episode of'].get('canonical title') or \
-                            self.data['episode of']['title']
-                return ser_title
+                ser_title = self.data['episode of']['title']
+                return canonicalTitle(ser_title)
             elif key == 'episode title':
-                return normalizeTitle(self.data.get('title', u''))
-            elif key == 'canonical episode title':
                 return self.data.get('title', u'')
+            elif key == 'canonical episode title':
+                return canonicalTitle(self.data.get('title', u''))
         if self.data.has_key('title'):
             if key == 'title':
-                return normalizeTitle(self.data['title'])
-            elif key == 'long imdb title':
-                return build_title(self.data, canonical=0)
-            elif key == 'canonical title':
                 return self.data['title']
+            elif key == 'long imdb title':
+                return build_title(self.data)
+            elif key == 'canonical title':
+                return canonicalTitle(self.data['title'])
             elif key == 'long imdb canonical title':
                 return build_title(self.data, canonical=1)
         return None
@@ -230,8 +230,8 @@ class Movie(_Container):
         if not isinstance(other, self.__class__): return 0
         if self.data.has_key('title') and \
                 other.data.has_key('title') and \
-                build_title(self.data, canonical=1) == \
-                build_title(other.data, canonical=1):
+                build_title(self.data, canonical=0) == \
+                build_title(other.data, canonical=0):
             return 1
         if self.accessSystem == other.accessSystem and \
                 self.movieID is not None and self.movieID == other.movieID:
@@ -244,16 +244,22 @@ class Movie(_Container):
         or if the the given Character is represented in this Movie."""
         from Person import Person
         from Character import Character
+        from Company import Company
         if isinstance(item, Person):
             for p in flatten(self.data, yieldDictKeys=1, scalar=Person,
-                            toDescend=(ListType, DictType, TupleType, Movie)):
+                            toDescend=(list, dict, tuple, Movie)):
                 if item.isSame(p):
                     return 1
         elif isinstance(item, Character):
             for p in flatten(self.data, yieldDictKeys=1, scalar=Person,
-                            toDescend=(ListType, DictType, TupleType, Movie)):
+                            toDescend=(list, dict, tuple, Movie)):
                 if item.isSame(p.currentRole):
                     return 1
+        elif isinstance(item, Company):
+            for c in flatten(self.data, yieldDictKeys=1, scalar=Company,
+                            toDescend=(list, dict, tuple, Movie)):
+                if item.isSame(c):
+                    return 1
         return 0
 
     def __deepcopy__(self, memo):
@@ -276,10 +282,10 @@ class Movie(_Container):
         if self.has_key('long imdb episode title'):
             title = self.get('long imdb episode title')
         else:
-            title = self.get('long imdb canonical title')
+            title = self.get('long imdb title')
         r = '<Movie id:%s[%s] title:_%s_>' % (self.movieID, self.accessSystem,
                                                 title)
-        if isinstance(r, UnicodeType): r = r.encode('utf_8', 'replace')
+        if isinstance(r, unicode): r = r.encode('utf_8', 'replace')
         return r
 
     def __str__(self):
@@ -293,51 +299,55 @@ class Movie(_Container):
     def summary(self):
         """Return a string with a pretty-printed summary for the movie."""
         if not self: return u''
-        def _nameAndRole(personList, joiner=', '):
+        def _nameAndRole(personList, joiner=u', '):
             """Build a pretty string with name and role."""
             nl = []
             for person in personList:
                 n = person.get('name', u'')
-                if person.currentRole: n += ' (%s)' % person.currentRole
+                if person.currentRole: n += u' (%s)' % person.currentRole
                 nl.append(n)
             return joiner.join(nl)
-        s = 'Movie\n=====\nTitle: %s\n' % \
+        s = u'Movie\n=====\nTitle: %s\n' % \
                     self.get('long imdb canonical title', u'')
         genres = self.get('genres')
-        if genres: s += 'Genres: %s.' % ', '.join(genres)
+        if genres: s += u'Genres: %s.\n' % u', '.join(genres)
         director = self.get('director')
         if director:
-            s += 'Director: %s.\n' % _nameAndRole(director)
+            s += u'Director: %s.\n' % _nameAndRole(director)
         writer = self.get('writer')
         if writer:
-            s += 'Writer: %s.\n' % _nameAndRole(writer)
+            s += u'Writer: %s.\n' % _nameAndRole(writer)
         cast = self.get('cast')
         if cast:
             cast = cast[:5]
-            s += 'Cast: %s.\n' % _nameAndRole(cast)
+            s += u'Cast: %s.\n' % _nameAndRole(cast)
         runtime = self.get('runtimes')
         if runtime:
-            s += 'Runtime: %s.\n' % ', '.join(runtime)
+            s += u'Runtime: %s.\n' % u', '.join(runtime)
         countries = self.get('countries')
         if countries:
-            s += 'Country: %s.\n' % ', '.join(countries)
+            s += u'Country: %s.\n' % u', '.join(countries)
         lang = self.get('languages')
         if lang:
-            s += 'Language: %s.\n' % ', '.join(lang)
+            s += u'Language: %s.\n' % u', '.join(lang)
         rating = self.get('rating')
         if rating:
-            s += 'Rating: %s' % rating
+            s += u'Rating: %s' % rating
             nr_votes = self.get('votes')
             if nr_votes:
-                s += '(%s votes)' % nr_votes
-            s += '.\n'
+                s += u' (%s votes)' % nr_votes
+            s += u'.\n'
         plot = self.get('plot')
+        if not plot:
+            plot = self.get('plot summary')
+            if plot:
+                plot = [plot]
         if plot:
             plot = plot[0]
             i = plot.find('::')
             if i != -1:
-                plot = plot[i+2:]
-            s += 'Plot: %s' % plot
+                plot = plot[:i]
+            s += u'Plot: %s' % plot
         return s
 
 
