@@ -4,7 +4,7 @@ helpers module (imdb package).
 This module provides functions not used directly by the imdb package,
 but useful for IMDbPY-based programs.
 
-Copyright 2006-2007 Davide Alberani <da@erlug.linux.it>
+Copyright 2006-2009 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import re
 from cgi import escape
-from types import UnicodeType, TupleType, ListType
 
 # The modClearRefs can be used to strip names and titles references from
 # the strings in Movie and Person objects.
@@ -35,8 +34,9 @@ from imdb import IMDb, imdbURL_movie_base, imdbURL_person_base, \
 from imdb.Movie import Movie
 from imdb.Person import Person
 from imdb.Character import Character
+from imdb.Company import Company
 from imdb.parser.http.utils import re_entcharrefssub, entcharrefs, \
-                                    entcharrefsget, subXMLRefs, subSGMLRefs
+                                    subXMLRefs, subSGMLRefs
 
 
 # An URL, more or less.
@@ -50,7 +50,7 @@ def makeCgiPrintEncoding(encoding):
         """Encode the given string using the %s encoding, and replace
         chars outside the given charset with XML char references.""" % encoding
         s = escape(s, quote=1)
-        if isinstance(s, UnicodeType):
+        if isinstance(s, unicode):
             s = s.encode(encoding, 'xmlcharrefreplace')
         return s
     return cgiPrint
@@ -63,14 +63,53 @@ re_subst = re.compile(r'%\((.+?)\)s')
 # Regular expression for <if condition>....</if condition> clauses.
 re_conditional = re.compile(r'<if\s+(.+?)\s*>(.+?)</if\s+\1\s*>')
 
+
+def makeTextNotes(replaceTxtNotes):
+    """Create a function useful to handle text[::optional_note] values.
+    replaceTxtNotes is a format string, which can include the following
+    values: %(text)s and %(notes)s.
+    Portions of the text can be conditionally excluded, if one of the
+    values is absent. E.g.: <if notes>[%(notes)s]</if notes> will be replaced
+    with '[notes]' if notes exists, or by an empty string otherwise.
+    The returned function is suitable be passed as applyToValues argument
+    of the makeObject2Txt function."""
+    def _replacer(s):
+        outS = replaceTxtNotes
+        if not isinstance(s, (unicode, str)):
+            return s
+        ssplit = s.split('::', 1)
+        text = ssplit[0]
+        # Used to keep track of text and note existence.
+        keysDict = {}
+        if text:
+            keysDict['text'] = True
+        outS = outS.replace('%(text)s', text)
+        if len(ssplit) == 2:
+            keysDict['notes'] = True
+            outS = outS.replace('%(notes)s', ssplit[1])
+        else:
+            outS = outS.replace('%(notes)s', u'')
+        def _excludeFalseConditionals(matchobj):
+            # Return an empty string if the conditional is false/empty.
+            if matchobj.group(1) in keysDict:
+                return matchobj.group(2)
+            return u''
+        while re_conditional.search(outS):
+            outS = re_conditional.sub(_excludeFalseConditionals, outS)
+        return outS
+    return _replacer
+
+
 def makeObject2Txt(movieTxt=None, personTxt=None, characterTxt=None,
-            joiner=' / ', applyToValues=lambda x: x, _recurse=True):
-    """"Return a function useful to pretty-print Movie, Person and
-    Character instances.
+               companyTxt=None, joiner=' / ',
+               applyToValues=lambda x: x, _recurse=True):
+    """"Return a function useful to pretty-print Movie, Person,
+    Character and Company instances.
 
     *movieTxt* -- how to format a Movie object.
     *personTxt* -- how to format a Person object.
     *characterTxt* -- how to format a Character object.
+    *companyTxt* -- how to format a Company object.
     *joiner* -- string used to join a list of objects.
     *applyToValues* -- function to apply to values.
     *_recurse* -- if True (default) manage only the given object.
@@ -82,18 +121,25 @@ def makeObject2Txt(movieTxt=None, personTxt=None, characterTxt=None,
         personTxt = '%(long imdb name)s'
     if characterTxt is None:
         characterTxt = '%(long imdb name)s'
+    if companyTxt is None:
+        companyTxt = '%(long imdb name)s'
     def object2txt(obj, _limitRecursion=None):
         """Pretty-print objects."""
         # Prevent unlimited recursion.
         if _limitRecursion is None:
             _limitRecursion = 0
         elif _limitRecursion > 5:
-            return ''
+            return u''
         _limitRecursion += 1
-        # XXX: recur also on dictionaries' keys and values?
         if isinstance(obj, (list, tuple)):
             return joiner.join([object2txt(o, _limitRecursion=_limitRecursion)
                                 for o in obj])
+        elif isinstance(obj, dict):
+            # XXX: not exactly nice, neither useful, I fear.
+            return joiner.join([u'%s::%s' %
+                            (object2txt(k, _limitRecursion=_limitRecursion),
+                            object2txt(v, _limitRecursion=_limitRecursion))
+                            for k, v in obj.items()])
         objData = {}
         if isinstance(obj, Movie):
             objData['movieID'] = obj.movieID
@@ -104,6 +150,9 @@ def makeObject2Txt(movieTxt=None, personTxt=None, characterTxt=None,
         elif isinstance(obj, Character):
             objData['characterID'] = obj.characterID
             outs = characterTxt
+        elif isinstance(obj, Company):
+            objData['companyID'] = obj.companyID
+            outs = companyTxt
         else:
             return obj
         def _excludeFalseConditionals(matchobj):
@@ -113,7 +162,7 @@ def makeObject2Txt(movieTxt=None, personTxt=None, characterTxt=None,
             if proceed:
                 return matchobj.group(2)
             else:
-                return ''
+                return u''
             return matchobj.group(2)
         while re_conditional.search(outs):
             outs = re_conditional.sub(_excludeFalseConditionals, outs)
@@ -131,7 +180,7 @@ def makeObject2Txt(movieTxt=None, personTxt=None, characterTxt=None,
                 value = u''
             elif not isinstance(value, (unicode, str)):
                 value = unicode(value)
-            outs = outs.replace('%(' + key + ')s', value)
+            outs = outs.replace(u'%(' + key + u')s', value)
         return outs
     return object2txt
 
@@ -246,7 +295,7 @@ def sortedEpisodes(m, season=None):
     if season is None:
         seasons = sortedSeasons(m)
     else:
-        if not isinstance(season, (TupleType, ListType)):
+        if not isinstance(season, (tuple, list)):
             seasons = [season]
     for s in seasons:
         eps_indx = m.get('episodes', {}).get(s, {}).keys()
@@ -257,11 +306,11 @@ def sortedEpisodes(m, season=None):
 
 
 # Idea and portions of the code courtesy of none none (dclist at gmail.com)
-_re_imdbIDurl = re.compile(r'\b(nm|tt|ch)([0-9]{7})\b')
+_re_imdbIDurl = re.compile(r'\b(nm|tt|ch|co)([0-9]{7})\b')
 def get_byURL(url, info=None, args=None, kwds=None):
-    """Return a Movie, Person or Character object for the given URL; info
-    is the info set to retrieve, args and kwds are respectively a list and
-    a dictionary or arguments to initialize the data access system.
+    """Return a Movie, Person, Character or Company object for the given URL;
+    info is the info set to retrieve, args and kwds are respectively a list
+    and a dictionary or arguments to initialize the data access system.
     Returns None if unable to correctly parse the url; can raise
     exceptions if unable to retrieve the data."""
     if args is None: args = []
@@ -278,6 +327,8 @@ def get_byURL(url, info=None, args=None, kwds=None):
         return ia.get_person(imdbID, info=info)
     elif imdbtype == 'ch':
         return ia.get_character(imdbID, info=info)
+    elif imdbtype == 'co':
+        return ia.get_company(imdbID, info=info)
     return None
 
 
