@@ -7,7 +7,8 @@ the imdb.IMDb function will return an instance of this class when
 called with the 'accessSystem' argument set to "http" or "web"
 or "html" (this is the default).
 
-Copyright 2004-2007 Davide Alberani <da@erlug.linux.it>
+Copyright 2004-2009 Davide Alberani <da@erlug.linux.it>
+               2008 H. Turgut Uyar <uyar@tekir.org>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,26 +26,39 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
 import sys
+import warnings
 from urllib import FancyURLopener, quote_plus
 from codecs import lookup
 
 from imdb import IMDbBase, imdbURL_movie_main, imdbURL_person_main, \
-        imdbURL_character_main, imdbURL_find
+                imdbURL_character_main, imdbURL_company_main, \
+                imdbURL_keyword_main, imdbURL_find, imdbURL_top250, \
+                imdbURL_bottom100
+from imdb.utils import analyze_title
 from imdb._exceptions import IMDbDataAccessError, IMDbParserError
 
 import searchMovieParser
 import searchPersonParser
 import searchCharacterParser
+import searchCompanyParser
+import searchKeywordParser
 import movieParser
 import personParser
 import characterParser
+import companyParser
+import topBottomParser
 
 
 class _ModuleProxy:
     """A proxy to instantiate and access parsers."""
-    def __init__(self, module, defaultKeys=None):
+    def __init__(self, module, defaultKeys=None, oldParsers=False,
+                useModule=None, fallBackToNew=False):
         """Initialize a proxy for the given module; defaultKeys, if set,
         muste be a dictionary of values to set for instanced objects."""
+        if oldParsers or fallBackToNew:
+            warnings.warn('The old set of parsers was removed; falling ' \
+                    'back to the new parsers.')
+        self.useModule = useModule
         if defaultKeys is None:
             defaultKeys = {}
         self._defaultKeys = defaultKeys
@@ -57,7 +71,11 @@ class _ModuleProxy:
         if name in _sm._OBJECTS:
             _entry = _sm._OBJECTS[name]
             # Initialize the parser.
-            obj = _entry[0]()
+            kwds = {}
+            if self.useModule:
+                kwds = {'useModule': self.useModule}
+            parserClass = _entry[0][0]
+            obj = parserClass(**kwds)
             attrsToSet = self._defaultKeys.copy()
             attrsToSet.update(_entry[1] or {})
             # Set attribute to the object.
@@ -84,6 +102,7 @@ _cookie_uu = 'su4/m8cho4c6HP+W1qgq6wchOmhnF0w+lIWvHjRUPJ6nRA9sccEafjGADJ6hQGrMd4
 class IMDbURLopener(FancyURLopener):
     """Fetch web pages and handle errors."""
     def __init__(self, *args, **kwargs):
+        self._last_url = u''
         FancyURLopener.__init__(self, *args, **kwargs)
         # Headers to add to every request.
         # XXX: IMDb's web server doesn't like urllib-based programs,
@@ -135,6 +154,7 @@ class IMDbURLopener(FancyURLopener):
             if PY_VERSION > (2, 3):
                 kwds['size'] = size
             content = uopener.read(**kwds)
+            self._last_url = uopener.url
             # Maybe the server is so nice to tell us the charset...
             server_encode = uopener.info().getparam('charset')
             # Otherwise, look at the content-type HTML meta tag.
@@ -168,7 +188,6 @@ class IMDbURLopener(FancyURLopener):
         if encode is None:
             encode = 'latin_1'
             # The detection of the encoding is error prone...
-            import warnings
             warnings.warn('Unable to detect the encoding of the retrieved '
                         'page [%s]; falling back to default latin1.' % encode)
         return unicode(content, encode, 'replace')
@@ -199,8 +218,9 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     accessSystem = 'http'
 
-    def __init__(self, isThin=0, adultSearch=1, proxy=-1,
-                *arguments, **keywords):
+    def __init__(self, isThin=0, adultSearch=1, proxy=-1, oldParsers=False,
+                fallBackToNew=False, useModule=None, cookie_id=-1,
+                cookie_uu=None, *arguments, **keywords):
         """Initialize the access system."""
         IMDbBase.__init__(self, *arguments, **keywords)
         self.urlOpener =  IMDbURLopener()
@@ -219,16 +239,49 @@ class IMDbHTTPAccessSystem(IMDbBase):
                 from imdb.utils import modNull
                 self._defModFunct = modNull
         self.do_adult_search(adultSearch)
+        if cookie_id != -1:
+            if cookie_id is None:
+                self.del_cookies()
+            elif cookie_uu is not None:
+                self.set_cookies(cookie_id, cookie_uu)
         if proxy != -1:
             self.set_proxy(proxy)
+        if useModule is not None:
+            if not isinstance(useModule, (list, tuple)) and ',' in useModule:
+                useModule = useModule.split(',')
         _def = {'_modFunct': self._defModFunct, '_as': self.accessSystem}
         # Proxy objects.
-        self.smProxy = _ModuleProxy(searchMovieParser, defaultKeys=_def)
-        self.spProxy = _ModuleProxy(searchPersonParser, defaultKeys=_def)
-        self.scProxy = _ModuleProxy(searchCharacterParser, defaultKeys=_def)
-        self.mProxy = _ModuleProxy(movieParser, defaultKeys=_def)
-        self.pProxy = _ModuleProxy(personParser, defaultKeys=_def)
-        self.cProxy = _ModuleProxy(characterParser, defaultKeys=_def)
+        self.smProxy = _ModuleProxy(searchMovieParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
+        self.spProxy = _ModuleProxy(searchPersonParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
+        self.scProxy = _ModuleProxy(searchCharacterParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
+        self.scompProxy = _ModuleProxy(searchCompanyParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
+        self.skProxy = _ModuleProxy(searchKeywordParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
+        self.mProxy = _ModuleProxy(movieParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
+        self.pProxy = _ModuleProxy(personParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
+        self.cProxy = _ModuleProxy(characterParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
+        self.compProxy = _ModuleProxy(companyParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
+        self.topBottomProxy = _ModuleProxy(topBottomParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
+
 
     def _normalize_movieID(self, movieID):
         """Normalize the given movieID."""
@@ -252,6 +305,14 @@ class IMDbHTTPAccessSystem(IMDbBase):
             raise IMDbParserError, 'invalid characterID "%s": %s' % \
                     (characterID, e)
 
+    def _normalize_companyID(self, companyID):
+        """Normalize the given companyID."""
+        try:
+            return '%07d' % int(companyID)
+        except ValueError, e:
+            raise IMDbParserError, 'invalid companyID "%s": %s' % \
+                    (companyID, e)
+
     def get_imdbMovieID(self, movieID):
         """Translate a movieID in an imdbID; in this implementation
         the movieID _is_ the imdbID.
@@ -270,6 +331,12 @@ class IMDbHTTPAccessSystem(IMDbBase):
         """
         return characterID
 
+    def get_imdbCompanyID(self, companyID):
+        """Translate a companyID in an imdbID; in this implementation
+        the companyID _is_ the imdbID.
+        """
+        return companyID
+
     def get_proxy(self):
         """Return the used proxy or an empty string."""
         return self.urlOpener.get_proxy()
@@ -284,6 +351,15 @@ class IMDbHTTPAccessSystem(IMDbBase):
         """
         self.urlOpener.set_proxy(proxy)
 
+    def set_cookies(self, cookie_id, cookie_uu):
+        """Set a cookie to access an IMDb's account."""
+        c_header = 'id=%s; uu=%s' % (cookie_id, cookie_uu)
+        self.urlOpener.set_header('Cookie', c_header)
+
+    def del_cookies(self):
+        """Remove the used cookie."""
+        self.urlOpener.del_header('Cookie')
+
     def do_adult_search(self, doAdult,
                         cookie_id=_cookie_id, cookie_uu=_cookie_uu):
         """If doAdult is true, 'adult' movies are included in the
@@ -291,27 +367,32 @@ class IMDbHTTPAccessSystem(IMDbBase):
         parameters to select a specific account (see your cookie
         or cookies.txt file."""
         if doAdult:
-            c_header = 'id=%s; uu=%s' % (cookie_id, cookie_uu)
-            self.urlOpener.set_header('Cookie', c_header)
+            self.set_cookies(cookie_id, cookie_uu)
+            #c_header = 'id=%s; uu=%s' % (cookie_id, cookie_uu)
+            #self.urlOpener.set_header('Cookie', c_header)
         else:
             self.urlOpener.del_header('Cookie')
 
     def _retrieve(self, url, size=-1):
         """Retrieve the given URL."""
+        ##print url
         return self.urlOpener.retrieve_unicode(url, size=size)
 
     def _get_search_content(self, kind, ton, results):
         """Retrieve the web page for a given search.
-        kind can be 'tt' (for titles), 'nm' (for names)
-        or 'char' (for characters).
+        kind can be 'tt' (for titles), 'nm' (for names),
+        'char' (for characters) or 'co' (for companies).
         ton is the title or the name to search.
         results is the maximum number of results to be retrieved."""
         if isinstance(ton, unicode):
             ton = ton.encode('utf-8')
         ##params = 'q=%s&%s=on&mx=%s' % (quote_plus(ton), kind, str(results))
         params = 's=%s;mx=%s;q=%s' % (kind, str(results), quote_plus(ton))
+        if kind == 'ep':
+            params = params.replace('s=ep;', 's=tt;ttype=ep;', 1)
         cont = self._retrieve(imdbURL_find % params)
-        if cont.find('more than 500 partial matches') == -1:
+        #print 'URL:', imdbURL_find % params
+        if cont.find('There were more than 500 partial matches') == -1:
             return cont
         # The retrieved page contains no results, because too many
         # titles or names contain the string we're looking for.
@@ -327,8 +408,14 @@ class IMDbHTTPAccessSystem(IMDbBase):
         ##params = 'q=%s&tt=on&mx=%s' % (quote_plus(title), str(results))
         ##cont = self._retrieve(imdbURL_find % params)
         cont = self._get_search_content('tt', title, results)
-        return self.smProxy.search_movie_parser.parse(cont,
-                                                    results=results)['data']
+        return self.smProxy.search_movie_parser.parse(cont, results=results)['data']
+
+    def _search_episode(self, title, results):
+        t_dict = analyze_title(title)
+        if t_dict['kind'] == 'episode':
+            title = t_dict['title']
+        cont = self._get_search_content('ep', title, results)
+        return self.smProxy.search_movie_parser.parse(cont, results=results)['data']
 
     def get_movie_main(self, movieID):
         if not self.isThin:
@@ -457,7 +544,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def get_movie_guests(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'epcast')
-        return self.mProxy.episodes_parser.parse(cont)
+        return self.mProxy.episodes_cast_parser.parse(cont)
     get_movie_episodes_cast = get_movie_guests
 
     def get_movie_merchandising_links(self, movieID):
@@ -497,6 +584,8 @@ class IMDbHTTPAccessSystem(IMDbBase):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'tvschedule')
         return self.mProxy.airing_parser.parse(cont)
 
+    get_movie_tv_schedule = get_movie_airing
+
     def get_movie_synopsis(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'synopsis')
         return self.mProxy.synopsis_parser.parse(cont)
@@ -513,8 +602,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
         #params = 'q=%s&nm=on&mx=%s' % (quote_plus(name), str(results))
         #cont = self._retrieve(imdbURL_find % params)
         cont = self._get_search_content('nm', name, results)
-        return self.spProxy.search_person_parser.parse(cont,
-                                                    results=results)['data']
+        return self.spProxy.search_person_parser.parse(cont, results=results)['data']
 
     def get_person_main(self, personID):
         cont = self._retrieve(imdbURL_person_main % personID + 'maindetails')
@@ -537,9 +625,9 @@ class IMDbHTTPAccessSystem(IMDbBase):
         cont = self._retrieve(imdbURL_person_main % personID + 'otherworks')
         return self.pProxy.otherworks_parser.parse(cont, getRefs=self._getRefs)
 
-    def get_person_agent(self, personID):
-        cont = self._retrieve(imdbURL_person_main % personID + 'agent')
-        return self.pProxy.agent_parser.parse(cont)
+    #def get_person_agent(self, personID):
+    #    cont = self._retrieve(imdbURL_person_main % personID + 'agent')
+    #    return self.pProxy.agent_parser.parse(cont)
 
     def get_person_publicity(self, personID):
         cont = self._retrieve(imdbURL_person_main % personID + 'publicity')
@@ -571,11 +659,10 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def _search_character(self, name, results):
         cont = self._get_search_content('char', name, results)
-        return self.scProxy.search_character_parser.parse(cont,
-                                                    results=results)['data']
+        return self.scProxy.search_character_parser.parse(cont, results=results)['data']
 
-    def get_character_main(self, personID):
-        cont = self._retrieve(imdbURL_character_main % personID)
+    def get_character_main(self, characterID):
+        cont = self._retrieve(imdbURL_character_main % characterID)
         ret = self.cProxy.character_main_parser.parse(cont)
         ret['info sets'] = ('main', 'filmography')
         return ret
@@ -597,4 +684,47 @@ class IMDbHTTPAccessSystem(IMDbBase):
         return self.cProxy.character_quotes_parser.parse(cont,
                                                     getRefs=self._getRefs)
 
+    def _search_company(self, name, results):
+        cont = self._get_search_content('co', name, results)
+        url = self.urlOpener._last_url
+        return self.scompProxy.search_company_parser.parse(cont, url=url,
+                                                    results=results)['data']
+
+    def get_company_main(self, companyID):
+        cont = self._retrieve(imdbURL_company_main % companyID)
+        ret = self.compProxy.company_main_parser.parse(cont)
+        return ret
+
+    def _search_keyword(self, keyword, results):
+        # XXX: the IMDb web server seems to have some serious problem with
+        #      non-ascii keyword.
+        #      E.g.: http://akas.imdb.com/keyword/fianc%E9/
+        #      will return a 500 Internal Server Error: Redirect Recursion.
+        keyword = keyword.encode('utf8', 'ignore')
+        try:
+            cont = self._get_search_content('kw', keyword, results)
+        except IMDbDataAccessError:
+            return []
+        return self.skProxy.search_keyword_parser.parse(cont, results=results)['data']
+
+    def _get_keyword(self, keyword, results):
+        keyword = keyword.encode('utf8', 'ignore')
+        try:
+            cont = self._retrieve(imdbURL_keyword_main % keyword)
+        except IMDbDataAccessError:
+            return []
+        return self.skProxy.search_moviekeyword_parser.parse(cont, results=results)['data']
+
+    def _get_top_bottom_movies(self, kind):
+        if kind == 'top':
+            parser = self.topBottomProxy.top250_parser
+            url = imdbURL_top250
+        elif kind == 'bottom':
+            parser = self.topBottomProxy.bottom100_parser
+            url = imdbURL_bottom100
+        else:
+            return []
+        cont = self._retrieve(url)
+        return parser.parse(cont)['data']
+
 
